%% MARQUISE post series: Postulates of quantum mechanics
% Quentin Stern - Mar 25, 2025
clear

%% Postulate 1

% Definition of kets for alpha and beta states
ket_alpha = [1; 0];
ket_beta  = [0; 1];

% Definition of an arbitrary linear combination 
% of alpha and beta
ket_psi = rand(2,1)+1i*rand(2,1);

% Normalization
ket_psi = ket_psi/norm(ket_psi);

bra_alpha = ket_alpha';
bra_beta  = ket_beta';
bra_psi   = ket_psi';

paa = abs(bra_alpha*ket_alpha)^2;
pab = abs(bra_alpha*ket_beta)^2;
pba = abs(bra_beta*ket_alpha)^2;
pbb = abs(bra_beta*ket_beta)^2;

p_alpha = abs(bra_alpha* ket_psi)^2;
p_beta  = abs(bra_beta* ket_psi)^2;

%% Postulate 2 & 3

% Definition of the angular momentum operators
Ix = [0  +1;+1  0]/2;
Iy = [0  -1;+1  0]*1i/2;
Iz = [+1  0; 0 -1]/2;

%% Postulate 4

% Definition of the angular momentum operators
Ix = [0  +1;+1  0]/2;
Iy = [0  -1;+1  0]*1i/2;
Iz = [+1  0; 0 -1]/2;

% Definition of states
alpha = [1; 0];          % Pure state
x     = [1; 1]/sqrt(2);  % Superposition
y     = [1; 1]/sqrt(2);  % Superposition
psi   = rand(2,1)+1i*rand(2,1);  % Arbitrary
psi   = psi/norm(psi);

% Expectations values of the alpha state all Cartesian coordinates
Ix_alpha = alpha'*Ix*alpha;
Iy_alpha = alpha'*Iy*alpha;
Iz_alpha = alpha'*Iz*alpha;

% Expectations values of the x state all Cartesian coordinates
Ix_x = x'*Ix*x;
Iy_x = x'*Iy*x;
Iz_x = x'*Iz*x;

% Expectations values of the y state all Cartesian coordinates
Ix_y = y'*Ix*y;
Iy_y = y'*Iy*y;
Iz_y = y'*Iz*y;

% Expectations values of the psi state all Cartesian coordinates
Ix_psi = psi'*Ix*psi;
Iy_psi = psi'*Iy*psi;
Iz_psi = psi'*Iz*psi;

%% Postulate 6 - Part 2

omega0  = 2*pi*1;   % Larmor freq (rad.s-1)
dt      = 0.01;     % Time increments (s)
K       = 300;      % Number of increments (-)

% Time axis and empty vectors to store results 
% of numerical propatation
t       = dt*(0:K-1);
It      = zeros(3,K);

% Initial state
psi0    = [1; 1]/sqrt(2);

% Angular momentum operators
Ix      = [0 1; 1 0]/2;
Iy      = [0 -1i; +1i 0]/2;
Iz      = [+1 0; 0 -1]/2;

% Hamiltonian and propagator during dt
H0      = omega0*Iz;
U0      = expm(-1i*H0*dt);

% Initialization of the system state
psi = psi0;

% Loop for propagation
for k = 1:K
    % Expectation values at time (i-1)*dt
    It(1,k) = real(psi'*Ix*psi);
    It(2,k) = real(psi'*Iy*psi);
    It(3,k) = real(psi'*Iz*psi);

    % Propagation during dt
    psi = U0*psi;
end

% Analytical solution
Ita = [cos(omega0*t)/2; sin(omega0*t)/2; zeros(1,K)];

% Plot of the results
figure(1), lw = 1.5;
plot(t,It,'LineWidth',lw), hold on
plot(t,Ita,'k--','LineWidth',lw), hold off
yline([+1 -1]/2)
legend({'{\itI_x}','{\itI_y}','{\itI_z}'})
xlabel('Time (s)'), ylabel('<{\itI_\mu}> (-)')
yticks((-1:+1)/2), yticklabels({'-1/2','0','+1/2'})
set(gca,'FontSize',12)